import es
from cfglib import AddonCFG
import gamethread
from playerlib import getPlayer, getPlayerList
from random import choice
from weaponlib import getWeaponList

ezrestrict = es.import_addon('ezrestrict')

info = es.AddonInfo()
info.name = 'LooneyOneInChamber'
info.basename = info.name.lower()
info.version = '1.2'
info.url = 'http://addons.eventscripts.com/addons/view/LooneyOneInChamber'
info.author = 'Satoon101'
es.ServerVar('loic_version', info.version,
                    info.name + ' Version').makepublic()

config = AddonCFG(es.getAddonPath(info.basename) + '/loic_config.cfg')

_usable_weapons = [weapon.basename for weapon in
        getWeaponList('#primary') + getWeaponList('#secondary')]

PlayerList = {}

class Player:
    def __init__(self, userid):
        self.userid = userid
        self.deaths = 0

    def playerSpawn(self):
        es.setplayerprop(self.userid, 'CBasePlayer.m_iHealth', 1)
        es.give(self.userid, 'player_weaponstrip')
        es.fire(self.userid, 'player_weaponstrip', 'strip')
        es.fire(self.userid, 'player_weaponstrip', 'Kill')
        gamethread.delayedname(0.1, 'give_weapons_%s'%self.userid, self.giveWeapons)

    def giveWeapons(self):
        es.give(self.userid, 'weapon_knife')
        es.give(self.userid, loic.current_weapon)
        gamethread.delayedname(0.1, 'set_ammo_%s'%self.userid, self.setAmmo)

    def setAmmo(self):
        player = getPlayer(self.userid)
        player.setAmmo(loic.current_weapon, 0)
        player.setClip(loic.current_weapon, 1)

    def cancelDelays(self):
        for name in ('give_weapons_%s', 'set_ammo_%s'):
            gamethread.cancelDelayed(name % self.userid)

    def playerDeath(self):
        self.deaths += 1
        if self.deaths < loic.current_lives:
            es.setplayerprop(self.userid, 'CCSPlayer.m_iPlayerState', 0)
            es.setplayerprop(self.userid, 'CBasePlayer.m_lifeState', 512)
            es.spawnplayer(self.userid)

    def playerKilled(self):
        player = getPlayer(self.userid)
        clip_amount = player.getClip(loic.current_weapon)
        if clip_amount < loic.current_maxammo:
            player.setClip(loic.current_weapon, clip_amount + 1)


class OneInChamberManagement:
    def __init__(self):
        self.roundOn = True
        self.weapon = config.cvar('loic_weapon_to_use', 'glock', 'Set to the weapon entity that all players will have')
        self.lives = config.cvar('loic_player_lives', 3, 'Maximum number of lives each player gets per round')
        self.maxammo = config.cvar('loic_max_bullets_allowed', 3, 'Maximum number of bullets any player can have at one time')
        self.grenades = {
            'flashbang':config.cvar('loic_allow_flashbang', 0, 'Enable/Disable players being allowed to have flashbangs'),
            'smokegrenade':config.cvar('loic_allow_smokegrenade', 0, 'Enable/Disable players being allowed to have smokegrenades')
                        }
        config.write()

        es.addons.registerClientCommandFilter(self.ccFilter)

    def roundStart(self):
        self.roundOn = True
        ezrestrict.restrict('@all', '#all')
        ezrestrict.unrestrict('@all', 'weapon_knife')
        self.current_weapon = str(self.weapon)
        if not self.current_weapon in _usable_weapons:
            self.current_weapon = choice(_usable_weapons)
        self.current_weapon = 'weapon_' + self.current_weapon
        ezrestrict.unrestrict('@all', self.current_weapon)
        for grenade in self.grenades:
            if int(self.grenades[grenade]):
                ezrestrict.unrestrict('@all', grenade)
        self.current_lives = int(self.lives)
        self.current_maxammo = int(self.maxammo)
        for player in getPlayerList('#alive'):
            getUser(str(player)).playerSpawn()

    def ccFilter(self, userid, args):
        if args[0] == 'drop':
            return False
        return True

    def unLoad(self):
        userid = es.getuserid()
        if userid:
            es.server.queuecmd('es_xfire %s func_buyzone Enable' % userid)
        es.addons.unregisterClientCommandFilter(self.ccFilter)

loic = OneInChamberManagement()


def getUser(userid):
    userid = str(userid)
    if not userid in PlayerList:
        PlayerList[userid] = Player(userid)
    return PlayerList[userid]


def load():
    config.execute()
    loic.roundStart()


def unload():
    global PlayerList
    loic.unLoad()
    for userid in PlayerList:
        getUser(userid).cancelDelays()
    del PlayerList


def es_map_start(ev):
    PlayerList.clear()


def round_start(ev):
    es.server.queuecmd('es_xfire %s func_buyzone Disable' % es.getuserid())
    loic.roundStart()
    for userid in es.getUseridList():
        getUser(userid).deaths = 0


def round_end(ev):
    loic.roundOn = False
    for userid in PlayerList:
        getUser(userid).cancelDelays()


def player_spawn(ev):
    if int(ev['es_userteam']) and loic.roundOn:
        getUser(ev['userid']).playerSpawn()


def player_death(ev):
    getUser(ev['userid']).playerDeath()
    attacker = ev['attacker']
    if int(attacker) and ev['es_userteam'] != ev['es_attackerteam']:
        getUser(attacker).playerKilled()


def player_disconnect(ev):
    userid = ev['userid']
    if userid in PlayerList:
        getUser(userid).cancelDelays()
        del PlayerList[userid]
